#include <LiquidCrystalRus.h>
#include <OneWire.h>
#include <EEPROM.h>

//byte v33 = 1;               // вход 3.3 V
byte butPin = 0;            // вход кнопок
int val;                    // знач. АЦП при нажатой кнопке
byte butPush = 0;           // код нажатой кнопки
boolean butRel=1;           // отпущена ли сейчас кнопка
int butval[8]={0,1023,141,503,742,0,325,0};   // АЦП кнопок
byte butsens=20;            // чувствительность кнопок
boolean keysound=0;         // включен ли звук кнопок
byte buzzer = 15;            // выход динамика
byte menu=0;                // текущий экран
byte P1_pin=13;              // порт на котором висит нагрузка (рабочий тэн)
byte P2_pin=16;              // порт на котором висит нагрузка (разгонный тэн)
byte P3_pin=11;              // порт на котором висит нагрузка (клапан воды)
boolean P1=0;               // статус порта (включен/выключен)
boolean P2=0;               // статус порта (включен/выключен)
boolean P3=0;               // статус порта (включен/выключен)
byte i;                     // счетчик
byte n;                     // счетчик
byte m=0;                   // счетчик
byte t=0;                   // счетчик
float T[]={0,0,0,0};        // переменная для хранения температуры
float Tp[]={0,0,0,0};       // переменная для хранения температуры
byte data[8];               // для хранения данных от датчика
unsigned long timer=0;      // таймер 
unsigned long procStTime=0; // время начала текущего процесса
unsigned long StTime=0;     // время запуска программы
byte Stop=0;                // причина останова
byte Step=0;                // шаг программы
boolean running = false;    // состояние программы
boolean is_h=1;             // отбирать ли головы
boolean is_b=1;             // отбирать ли тело(хвосты)
float Tsam=70;              // max температура продукта
float Th=89;                // температура окончания отбора голов
float Tb=96;                // температура начала отбора хвостов
float Ts=99;                // температура при которой останавливаем процесс
float Tw=60;                // max температура охлаждающей воды
float Tc=80;                // температура при которой включаем воду
float Tr=75;                // температура отключения разгона

byte TotalDS=0;   // число найденых датчиков на линии
byte addr[4][8] = {
   {0,0,0,0,0,0,0,0},
   {0,0,0,0,0,0,0,0},
   {0,0,0,0,0,0,0,0},
   {0,0,0,0,0,0,0,0},
};  

OneWire  ds(3);            // на этот пин подключен DS
LiquidCrystalRus lcd(8, 9, 4, 5, 6, 7); //инициализация LCD

void setup()
{
  lcd.begin(2, 16);
//  Serial.begin(9600);
  pinMode(P1_pin, OUTPUT);
  pinMode(P2_pin, OUTPUT);
  pinMode(P3_pin, OUTPUT);
  pinMode(buzzer, OUTPUT);
  if (analogRead(butPin) < 1000) { // процедура калибровки
    lcd.print("Keyboard setup");
    lcd.setCursor(0,1);
    delay(2000);
    for ( i = 1; i < 6; i++) {
    lcd.clear();
    lcd.print("Press ");
    lcd.print(i, DEC);
    while (analogRead(butPin) > 1000) {};
    button ();
    butval[i]=val;
    lcd.clear();
    lcd.setCursor(0,1);
    lcd.print(i, DEC);
    lcd.print(" ");
    lcd.print(butval[i], DEC);
    EEPROM.write(224+i*2,butval[i]/4); // по 2 байта на значение начиная с 224
    EEPROM.write(225+i*2,butval[i]%4);
    delay(1000);
    }
  } // конец калибровки
  else { // читаем АЦП кнопок
    for ( i = 0; i < 8; i++) {
      butval[i] = EEPROM.read(224+i*2)*4+EEPROM.read(225+i*2);
    }
  }
  read_DS();  //считываем адреса датчиков из памяти
  keysound = EEPROM.read(241);
  butsens = EEPROM.read(242); if (butsens < 5 || butsens > 100) { butsens = 20;}
  tone(buzzer, 5000, 10000); // включаем пищалку. Это сигнализация на случай если пропадет и снова появится питание
  
}

void loop()
{
 
    ds.reset();
    ds.write(0xCC,1);          // пропуск ROM
    ds.write(0x44,1);          // запускаем конвертацию
    for( n = 1; n < 90; n++) { //ждем 900 мс
      delay(10);            
      if (analogRead(butPin) < 1000 && butRel) { Menu(button ()); } //периодически проверяя не нажата ли кнопка
      if (analogRead(butPin) > 1000) {butRel=1;}
    }
    for( n = 0; n < 4; n++) {
      ds.reset();
      ds.select(addr[n]);
      ds.write(0xBE);         // считываем ОЗУ датчика
      for ( i = 0; i < 9; i++) {data[i] = ds.read();} // обрабатываем 9 байт
      Tp[n] = float((data[1] << 8) + data[0])/16;     // высчитываем температуру :)
      if (Tp[n] == 85) {} else {T[n]=Tp[n];}         // это для того, чтобы при глюке датчика (обычно при переключениях 
        // выдача температуры 85 град не приводила к переходу на другой шаг программы
      }
    timer=millis()/1000;         // текущее время в сек.
  if (Stop) {running=0;}
  if (running) {
    if (!is_h && Step==0) {Step++;}
    if (!is_b && Step==1) {Step++;}
    P1=1;
    if (T[2] > Tsam) {Stop=1;} // нет охлаждения
    if (T[0] > Th && Step==0) {Stop=4; tone(buzzer, 1000, 500); delay(200);tone(buzzer, 2000, 1000);} // отобрали головы
    if (T[0] > Tb && Step==1) {Stop=4; tone(buzzer, 2000, 500);delay(200); tone(buzzer, 1000, 500);} // отобрали тело
    if (T[0] > Ts) {Stop=0; running=0; tone(buzzer, 500, 3000);delay(200); tone(buzzer, 500, 3000);} // все закончили (нормальный останов)
    if (T[3] > Tw) {Stop=6;} // высокая Т охлаждающей воды
    if (T[1] > Tc || T[0] > Tr) {P3=1;} else {P3=0;} // вода
    if (T[0] < Tr) {P2=1;} else {P2=0;} // разгон
  }
  else {
  P1=0; 
  P2=0; 
  P3=0; 
  }
  digitalWrite(P1_pin,P1); 
  digitalWrite(P2_pin,P2); 
  digitalWrite(P3_pin,P3); 
  
  Menu(0);
}

/*int button () {
    int exitMenu = 1;          // выход из меню
    butPush = 0; // код нажатой кнопки
    if (keysound) {tone(buzzer, 1000);}
    delay(15);  // задержка для устранения дребезга контактов
    val = analogRead(butPin);  // читаем АЦП порта кнопок (значение от 0 до 1023)
    if (val > (butval[5]-butsens) && val < (butval[5]+butsens)) {butPush = 5;}  
    if (val > (butval[4]-butsens) && val < (butval[4]+butsens)) {butPush = 4;}
    if (val > (butval[3]-butsens) && val < (butval[3]+butsens)) {butPush = 3;}
    if (val > (butval[2]-butsens) && val < (butval[2]+butsens)) {butPush = 2;}
    if (val > (butval[1]-butsens)) {butPush = 1;}
    butRel=0;
    noTone(buzzer);
  return butPush;
}
*/
int button () {                // подпрограмма считывания кнопок
    int exitMenu = 1;          // если exitMenu истина то выходим из меню
    butPush = 0;               // код нажатой кнопки
	 if (keysound) {tone(buzzer, 400, 30);}
    delay(15);                 // задержка для устранения дребезга контактов
    val = analogRead(butPin);  // читаем АЦП порта кнопок (значение от 0 до 1023)
    if (val > (butval[5]-butsens) && val < (butval[5]+butsens)) {butPush = 5;}  // если считанное значение около (на величину butsens)
    // значения соответствующего кнопке 5 то считаем что она и нажата
    if (val > (butval[4]-butsens) && val < (butval[4]+butsens)) {butPush = 4;}  // аналогично для остальных
    if (val > (butval[3]-butsens) && val < (butval[3]+butsens)) {butPush = 3;}
    if (val > (butval[2]-butsens) && val < (butval[2]+butsens)) {butPush = 2;}
    if (val > (butval[1]-butsens) && val < (butval[1]+butsens)) {butPush = 1;}
//    if (val < (butval[1]+butsens)) {butPush = 1;}     
    // коды кнопок: 1 - вверх, 2 - влево, 3 - ок, 4 - вправо, 5 вниз 
    butRel=0;                  // эта переменная нужна для проверки в главной программе
    return butPush;            // возвращаем значение нажатой кнопки
}

void printTimer () {
         timer=timer-StTime;
         n=timer/60;
         if (n < 100) lcd.print(" "); 
         if (n < 10) lcd.print(" "); 
         lcd.print(n,DEC);
         lcd.print(":"); 
         n=timer-(timer/60)*60;
         if (n < 10) lcd.print("0"); 
         lcd.print(n,DEC);    
}

byte read_DS() {
  for (n=0; n<4; n++) {
    for (i=0; i<8; i++) {
      addr[n][i] = EEPROM.read(128+n*8+i);
    }
  }
  return 0;
}

byte write_DS() {
  for (i=0; i<8; i++) {
    if (m==0) {EEPROM.write(128+t*8+i,0);}
    else {EEPROM.write(128+t*8+i,addr[m-1][i]);}
    }
  return 0;
}

void Menu(int butPush) {
    lcd.clear();
    if (Stop) {menu=1;}
    switch (menu) {
      case 0: // температуры
        lcd.print("1:");
        lcd.print(T[0]);
        lcd.setCursor(8,0);
        lcd.print("2:");
        lcd.print(T[1]);
        lcd.setCursor(0,1);
        lcd.print("3:");
        lcd.print(T[2]);
        lcd.setCursor(8,1);
        lcd.print("4:");
        lcd.print(T[3]);
        if (butPush == 1) {menu=3; Menu(0);}
        if (butPush == 5) {menu++; Menu(0);}
      break;

      case 1: // статус портов
        if (Stop) {
          lcd.print("Stoped"); //Ocтaнoвлeнo
          lcd.setCursor(0,1);
          lcd.print("Error "); //Oшибкa
          lcd.print(Stop, DEC);
          if (butPush == 2) {Stop=0; Menu(0);}
          if (butPush == 4) {Stop=0; running=1; Step++; Menu(0);}
        }
        else {
        if (P1) {lcd.print(" T1");}
        if (P2) {lcd.setCursor(6,0); lcd.print("T2+");}
        if (P3) {lcd.setCursor(11,0); lcd.print("WATER");}
        lcd.setCursor(0,1);
        lcd.print(T[0]); lcd.setCursor(5,1); lcd.print("ИC"); //если не русский дисплей будет "градус Цельсия" )
        lcd.setCursor(10,1); 
        if (running) {printTimer();} else {lcd.print("OFF");}
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 5) {menu++; Menu(0);}
        }
      break;
      
      case 2: // запуск процесса
        if (running) {
          lcd.print("Process Started"); // Пpoцecc зaпущeн
          lcd.setCursor(0,1);
          timer=timer-StTime;
          lcd.print(timer/60);
          lcd.print(":"); 
          n=timer-(timer/60)*60;
          if (n < 10) lcd.print("0"); 
          lcd.print(n,DEC);   
         }
        else {lcd.print("Stoped");} // Ocтaнoвлeнo
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 3) {
        if (running) {running=0; Menu(0);}
        else {running=1; Step=0; StTime=timer;  Menu(0);}    }
        if (butPush == 5) {menu++; Menu(0);}
      break;      

      case 3: // меню настроек
//        lcd.print("Hacтpoйки");
        lcd.print("Setup"); //Настройки
        lcd.setCursor(0,1);
        lcd.print("Press  OK");
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 3) {menu=40; Menu(0);}
        if (butPush == 5) {menu=0; Menu(0);}
      break;      
      
//////////////////////////////////////////////////////////////////////////////////////

      case 40: // max T продукта 
        lcd.print("Max T SAMOGON"); //Max T пpoдуктa
        lcd.setCursor(0,1);
        lcd.print(Tsam, DEC);
        if (butPush == 1) {menu=55; Menu(0);}
        if (butPush == 2) {Tsam=Tsam-0.25; Menu(0);}
        if (butPush == 4) {Tsam=Tsam+0.25; Menu(0);}
        if (butPush == 5) {menu++; Menu(0);}
      break;
      case 41: // отбирать головы?
        lcd.print("Get HEAD?"); //Отбирать гoлoвы?
        lcd.setCursor(0,1);
        if (is_h) {lcd.print("     Yes");} else {lcd.print("      No");}
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 2) {is_h=0; Menu(0);}
        if (butPush == 4) {is_h=1; Menu(0);}
        if (butPush == 5) {menu++; Menu(0);}
      break;
      case 42: // T отбора голов
        lcd.print("T end of HEAD"); //T конеч. oтб. г.
        lcd.setCursor(0,1);
        lcd.print(Th, DEC);
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 2) {Th=Th-0.25; Menu(0);}
        if (butPush == 4) {Th=Th+0.25; Menu(0);}
        if (butPush == 5) {menu++; Menu(0);}
      break;
      case 43: // отбирать хвосты?
        lcd.print("Get XVOST?"); //Отбирать хвосты?
        lcd.setCursor(0,1);
        if (is_b) {lcd.print("      Yes");} else {lcd.print("      No");}
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 2) {is_b=0; Menu(0);}
        if (butPush == 4) {is_b=1; Menu(0);}
        if (butPush == 5) {menu++; Menu(0);}
      break;
      case 44: // Т отбора тела
        lcd.print("T start get XVOST"); //T нач. oтб. хв.
        lcd.setCursor(0,1);
        lcd.print(Tb, DEC);
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 2) {Tb=Tb-0.25; Menu(0);}
        if (butPush == 4) {Tb=Tb+0.25; Menu(0);}
        if (butPush == 5) {menu++; Menu(0);}
      break;
      case 45: // Т останова процесса
        lcd.print("T Stoped"); //T остановки
        lcd.setCursor(0,1);
        lcd.print(Ts, DEC);
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 2) {Ts=Ts-0.25; Menu(0);}
        if (butPush == 4) {Ts=Ts+0.25; Menu(0);}
        if (butPush == 5) {menu++; Menu(0);}
      break;
      case 46: // max T охл. воды
        lcd.print("T max cool water"); //T охл. воды
        lcd.setCursor(0,1);
        lcd.print(Tw, DEC);
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 2) {Tw=Tw-0.25; Menu(0);}
        if (butPush == 4) {Tw=Tw+0.25; Menu(0);}
        if (butPush == 5) {menu++; Menu(0);}
      break;
      case 47: // T отключения разгона
        lcd.print("T off + HEAT"); //T откл. разгона
        lcd.setCursor(0,1);
        lcd.print(Tr, DEC);
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 2) {Tr=Tr-0.25; Menu(0);}
        if (butPush == 4) {Tr=Tr+0.25; Menu(0);}
        if (butPush == 5) {menu++; Menu(0);}
      break;
      case 48: // T включения охлаждения
          lcd.print("T ON cool WATER "); // T включения охлаждения
        lcd.setCursor(0,1);
        lcd.print(Tc, DEC);
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 2) {Tc=Tc-0.25; Menu(0);}
        if (butPush == 4) {Tc=Tc+0.25; Menu(0);}
        if (butPush == 5) {menu++; Menu(0);}
      break;
      case 49: // 
        lcd.print("Rezerv 2");
        lcd.setCursor(0,1);
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 5) {menu++; Menu(0);}
      break;
      case 50: // 
        lcd.print("Rezerv 3");
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 5) {menu++; Menu(0);}
      break;
      case 51: // напряжение питания
        lcd.print("Supply voltage"); //Напр. питания
        lcd.setCursor(0,1);
        lcd.print(1024*1.1/(analogRead(14)),DEC);
    //    Serial.print(analogRead(14),DEC);
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 5) {menu++; Menu(0);}
      break;
      case 52: // Звук кнопок
        lcd.print("Key beep"); //Звук кнопок
        lcd.setCursor(0,1);
        if (keysound) {lcd.print("    ON");}
        else  {lcd.print("   OFF");}
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 2 || butPush == 4) {keysound=!keysound; Menu(0); EEPROM.write(241,keysound);}
        if (butPush == 5) {menu++; Menu(0);}
      break;      
      case 53: // Чувств. кнопок
        lcd.print("Key sensitivity"); //Чувствит. кнопок
        lcd.setCursor(0,1);
        lcd.print(butsens, DEC);
        if (butPush == 1) {menu--; EEPROM.write(242,butsens); Menu(0);}
        if (butPush == 2) {butsens--; if (butsens<10) {butsens=10;} Menu(0);}
        if (butPush == 4) {butsens++; if (butsens>100) {butsens=100;}Menu(0);}                
        if (butPush == 5) {menu++; EEPROM.write(242,butsens); Menu(0);}
      break;      

      case 54: // поиск датчиков
        i = 0;
        n=0;
        while (i != 1 ) /// ищем датчики 
        {
          if ( ds.search(addr[n]) != 1) {
            ds.reset_search();
            i = 1; }
          else { 
            n++; 
            TotalDS = n;
          }
        }
        lcd.print("Search DS"); //Поиск датчиков
        lcd.setCursor(0,1);
        lcd.print(n, DEC);
        lcd.print(" searching"); //Найдено
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 4) {menu=100; Menu(0);}
        if (butPush == 5) {menu++; Menu(0);}
      break;


      case 55: // выход из меню
        lcd.print("Exit Menu"); //Выйти из меню
        lcd.setCursor(0,1);
        lcd.print("OK for Exit"); //OK для выхода
        if (butPush == 1) {menu--; Menu(0);}
        if (butPush == 3) {menu=1; Menu(0);}
        if (butPush == 5) {menu=40; Menu(0);}
      break;      
      
      case 100: // меню настроек датчиков DS1820
        lcd.print("T");
        lcd.print(t+1,DEC);
        lcd.setCursor(0,1);
        if (m==0) {lcd.print("Not present");} //Отсутстует
           else {lcd.print(T[m-1],DEC);}
        if (butPush == 1) {if (t==0) {menu=101;} else {t--;} Menu(0);}
        if (butPush == 2) {if (m==0) {} else {m--;} Menu(0);}
        if (butPush == 3) {write_DS(); lcd.setCursor(0,1); lcd.print("Write in EEPROM"); tone(buzzer, 800, 1000);  menu=100; Menu(0);} //записано в eeprom
        if (butPush == 4) {m++; if (m>(TotalDS)) {m=TotalDS;} Menu(0);}
        if (butPush == 5) {if (t==3) {menu=101;} else {t++;}  Menu(0);}
      break; 

      case 101: // Выход из меню
        lcd.print("Exit Menu"); //Выход из меню
        lcd.setCursor(0,1);
        lcd.print("OK for Exit"); //OK для выхода
        if (butPush == 1) {t=3; menu=100; Menu(0);}
        if (butPush == 3) {menu=52; Menu(0);}
        if (butPush == 5) {t=0; menu=100; Menu(0);}  
      break;      


    } // end switch

}

